(function (webapi, $) {
    function safeAjax(ajaxOptions) {
        var deferredAjax = $.Deferred();

        shell.getTokenDeferred().done(function (token) {
            // add headers for ajax
            if (!ajaxOptions.headers) {
                $.extend(ajaxOptions, {
                    headers: {
                        "__RequestVerificationToken": token
                    }
                });
            } else {
                ajaxOptions.headers["__RequestVerificationToken"] = token;
            }
            $.ajax(ajaxOptions)
                .done(function (data, textStatus, jqXHR) {
                    validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
                }).fail(deferredAjax.reject); //ajax
        }).fail(function () {
            deferredAjax.rejectWith(this, arguments); // on token failure, pass the token ajax and args
        });

        return deferredAjax.promise();
    }
    webapi.safeAjax = safeAjax;
})(window.webapi = window.webapi || {}, jQuery);


window.createRecord = function (entityObj, actionUrl) {

    return new Promise(function (resolve, reject) {
        webapi.safeAjax({
            type: "POST",
            url: actionUrl,
            contentType: "application/json",
            data: JSON.stringify(entityObj),
            success: function (res, status, xhr) {
                resolve(xhr.getResponseHeader('entityid'));
            },
            error: function (e) {
                reject(e);
            }
        });
    });
};


window.updateRecordAttribute = function (paramsObj, actionUrl) {

    return new Promise(function (resolve, reject) {
        webapi.safeAjax({
            type: "PUT",
            url: actionUrl,
            contentType: "application/json",
            data: JSON.stringify(paramsObj),
            success: function (res) {
                resolve(res);
            },
            error: function (e) {
                reject(e);
            }
        });
    });
};


window.updateRecord = function (paramsObj, actionUrl) {

    return new Promise(function (resolve, reject) {
        webapi.safeAjax({
            type: "PATCH",
            url: actionUrl,
            contentType: "application/json",
            data: JSON.stringify(paramsObj),
            success: function (res) {
                resolve(res);
            },
            error: function (e) {
                reject(e);
            }
        });
    });
};

window.deleteRecord = function (paramsObj, actionUrl) {

    return new Promise(function (resolve, reject) {
        webapi.safeAjax({
            type: "DELETE",
            url: actionUrl,
            contentType: "application/json",
            success: function (res) {
                resolve(res);
            },
            error: function (e) {
                reject(e);
            }
        });
    });
};

//attributes name needs to be passed comma seperated
window.retrieveRecord = function (attributes,actionUrl) {
    actionUrl = actionUrl + "?$select=" + attributes;
    return new Promise(function (resolve, reject) {
        webapi.safeAjax({
            type: "GET",
            url: actionUrl,
            contentType: "application/json",
            success: function (res) {
                resolve(res);
            },
            error: function (e) {
                reject(e);
            }
        });
    });
};

